import argparse

from cyberdrop_dl import __version__ as VERSION
from cyberdrop_dl.utils.dataclasses.supported_domains import SupportedDomains


def parse_args() -> argparse.Namespace:
    """Parses the command line arguments passed into the program"""
    parser = argparse.ArgumentParser(description="Bulk downloader for multiple file hosts")
    general = parser.add_argument_group("General")
    general.add_argument("-V", "--version", action="version", version=f"%(prog)s {VERSION}")
    general.add_argument("--config", type=str, help="name of config to load", default="")
    general.add_argument("--download", action="store_true", help="Skip the UI and go straight to downloading", default=False)
    general.add_argument("--download-all-configs", action="store_true", help="Skip the UI and go straight to downloading (runs all configs sequentially)", default=False)
    general.add_argument("--retry-failed", action="store_true", help="retry failed downloads", default=False)

    # File Paths
    file_paths = parser.add_argument_group("File_Paths")
    file_paths.add_argument("-i", "--input-file", type=str, help="path to txt file containing urls to download", default="")
    file_paths.add_argument("-d", "--output-folder", type=str, help="path to download folder", default="")
    file_paths.add_argument("--config-file", type=str, help="path to the CDL settings.yaml file to load", default="")
    file_paths.add_argument("--appdata-folder", type=str, help="path to where you want CDL to store it's AppData folder", default="")

    # Settings
    download_options = parser.add_argument_group("Download_Options")
    download_options.add_argument("--block-download-sub-folders", action="store_true", help="block sub folder creation", default=False)
    download_options.add_argument("--disable-download-attempt-limit", action="store_true", help="disable download attempt limit", default=False)
    download_options.add_argument("--disable-file-timestamps", action="store_true", help="disable file timestamps", default=False)
    download_options.add_argument("--include-album-id-in-folder-name", action="store_true", help="include album id in folder name", default=False)
    download_options.add_argument("--include-thread-id-in-folder-name", action="store_true", help="include thread id in folder name", default=False)
    download_options.add_argument("--remove-domains-from-folder-names", action="store_true", help="remove website domains from folder names", default=False)
    download_options.add_argument("--remove-generated-id-from-filenames", action="store_true", help="remove site generated id from filenames", default=False)
    download_options.add_argument("--skip-download-mark-completed", action="store_true", help="skip download and mark as completed in history", default=False)

    file_size_limits = parser.add_argument_group("File_Size_Limits")
    file_size_limits.add_argument("--maximum-image-size", type=int, help="maximum image size in bytes (default: %(default)s)", default=0)
    file_size_limits.add_argument("--maximum-video-size", type=int, help="maximum video size in bytes (default: %(default)s)", default=0)
    file_size_limits.add_argument("--maximum-other-size", type=int, help="maximum other size in bytes (default: %(default)s)", default=0)
    file_size_limits.add_argument("--minimum-image-size", type=int, help="minimum image size in bytes (default: %(default)s)", default=0)
    file_size_limits.add_argument("--minimum-video-size", type=int, help="minimum video size in bytes (default: %(default)s)", default=0)
    file_size_limits.add_argument("--minimum-other-size", type=int, help="minimum other size in bytes (default: %(default)s)", default=0)

    ignore_options = parser.add_argument_group("Ignore_Options")
    ignore_options.add_argument("--exclude-videos", action="store_true", help="exclude videos from downloading", default=False)
    ignore_options.add_argument("--exclude-images", action="store_true", help="exclude images from downloading", default=False)
    ignore_options.add_argument("--exclude-audio", action="store_true", help="exclude images from downloading", default=False)
    ignore_options.add_argument("--exclude-other", action="store_true", help="exclude other files from downloading", default=False)
    ignore_options.add_argument("--ignore-coomer-ads", action="store_true", help="ignore coomer ads when scraping", default=False)
    ignore_options.add_argument("--skip-hosts", choices=SupportedDomains.supported_hosts, action="append", help="skip these domains when scraping", default=[])
    ignore_options.add_argument("--only-hosts", choices=SupportedDomains.supported_hosts, action="append", help="only scrape these domains", default=[])

    runtime_options = parser.add_argument_group("Runtime_Options")
    runtime_options.add_argument("--ignore-history", action="store_true", help="ignore history when scraping", default=False)
    runtime_options.add_argument("--skip-check-for-partial-files", action="store_true", help="skip check for partial downloads", default=False)
    runtime_options.add_argument("--skip-check-for-empty-folders", action="store_true", help="skip check (and removal) for empty folders", default=False)
    runtime_options.add_argument("--delete-partial-files", action="store_true", help="delete partial downloads", default=False)
    runtime_options.add_argument("--send-unsupported-to-jdownloader", action="store_true", help="send unsupported urls to jdownloader", default=False)

    # Links
    parser.add_argument("links", metavar="link", nargs="*", help="link to content to download (passing multiple links is supported)", default=[])
    return parser.parse_args()



from __future__ import annotations

import json
from typing import TYPE_CHECKING

from aiolimiter import AsyncLimiter
from yarl import URL

from cyberdrop_dl.scraper.crawler import Crawler
from cyberdrop_dl.utils.dataclasses.url_objects import ScrapeItem
from cyberdrop_dl.utils.utilities import error_handling_wrapper, log, get_filename_and_ext

if TYPE_CHECKING:
    from cyberdrop_dl.managers.manager import Manager


class ScrolllerCrawler(Crawler):
    def __init__(self, manager: Manager):
        super().__init__(manager, "scrolller", "Scrolller")
        self.scrolller_api = URL("https://api.scrolller.com/api/v2/graphql")
        self.headers = {"Content-Type": "application/json"}
        self.request_limiter = AsyncLimiter(10, 1)

    """~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"""

    async def fetch(self, scrape_item: ScrapeItem) -> None:
        """Determines where to send the scrape item based on the url"""
        task_id = await self.scraping_progress.add_task(scrape_item.url)

        if "r" in scrape_item.url.parts:
            await self.subreddit(scrape_item)
        else:
            await log(f"Unsupported URL: {scrape_item.url}")
            await self.manager.log_manager.write_unsupported_urls_log(scrape_item.url)

        await self.scraping_progress.remove_task(task_id)

    @error_handling_wrapper
    async def subreddit(self, scrape_item: ScrapeItem) -> None:
        """Scrapes an album"""
        subreddit = scrape_item.url.parts[-1]
        title = await self.create_title(subreddit, None, None)
        await scrape_item.add_to_parent_title(title)
        scrape_item.part_of_album = True

        request_body = {
                "query": """
                    query SubredditQuery(
                        $url: String!
                        $filter: SubredditPostFilter
                        $iterator: String
                    ) {
                        getSubreddit(url: $url) {
                            title
                            children(
                                limit: 10000
                                iterator: $iterator
                                filter: $filter
                                disabledHosts: null
                            ) {
                                iterator
                                items {
                                    title
                                    mediaSources {
                                        url
                                    }
                                    blurredMediaSources {
                                        url
                                    }
                                }
                            }
                        }
                    }
                """,
                "variables": {
                    "url": f"/r/{subreddit}",
                    "filter": None,
                    "hostsDown": None
                },
            }

        iterator = None
        iterations = 0

        while True:
            request_body["variables"]["iterator"] = iterator
            data = await self.client.post_data(self.domain, self.scrolller_api, data=json.dumps(request_body))

            if data:
                items = data["data"]["getSubreddit"]["children"]["items"]

                for item in items:
                    media_sources = item['mediaSources']
                    if media_sources:
                        highest_res_image_url = URL(media_sources[-1]['url'])
                        filename, ext = await get_filename_and_ext(highest_res_image_url.name)
                        await self.handle_file(highest_res_image_url, scrape_item, filename, ext)

                prev_iterator = iterator
                iterator = data["data"]["getSubreddit"]["children"]["iterator"]

                if not items or iterator == prev_iterator:
                    break
                if iterations > 0 and iterator is None:
                    break
            else:
                break

            iterations += 1



from __future__ import annotations

from typing import TYPE_CHECKING

from aiolimiter import AsyncLimiter
from yarl import URL

from cyberdrop_dl.scraper.crawler import Crawler
from cyberdrop_dl.utils.dataclasses.url_objects import ScrapeItem
from cyberdrop_dl.utils.utilities import get_filename_and_ext, error_handling_wrapper

if TYPE_CHECKING:
    from cyberdrop_dl.managers.manager import Manager


class FapelloCrawler(Crawler):
    def __init__(self, manager: Manager):
        super().__init__(manager, "fapello", "Fapello")
        self.request_limiter = AsyncLimiter(5, 1)

    """~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"""

    async def fetch(self, scrape_item: ScrapeItem) -> None:
        """Determines where to send the scrape item based on the url"""
        task_id = await self.scraping_progress.add_task(scrape_item.url)

        if not str(scrape_item.url).endswith("/"):
            scrape_item.url = URL(str(scrape_item.url) + "/")

        if scrape_item.url.parts[-2].isnumeric():
            await self.post(scrape_item)
        else:
            await self.profile(scrape_item)

        await self.scraping_progress.remove_task(task_id)

    @error_handling_wrapper
    async def profile(self, scrape_item: ScrapeItem) -> None:
        """Scrapes a profile"""
        async with self.request_limiter:
            soup, response_url = await self.client.get_BS4_and_return_URL(self.domain, scrape_item.url)
            if response_url != scrape_item.url:
                return

        title = await self.create_title(soup.select_one('h2[class="font-semibold lg:text-2xl text-lg mb-2 mt-4"]').get_text(), None, None)

        content = soup.select("div[id=content] a")
        for post in content:
            if "javascript" in post.get('href'):
                video_tag = post.select_one('iframe')
                video_link = URL(video_tag.get('src'))
                new_scrape_item = await self.create_scrape_item(scrape_item, video_link, "", True)
                await self.manager.queue_manager.url_objects_to_map.put(new_scrape_item)
            else:
                link = URL(post.get('href'))
                new_scrape_item = await self.create_scrape_item(scrape_item, link, title, True)
                await self.scraper_queue.put(new_scrape_item)

        next_page = soup.select_one('div[id="next_page"] a')
        if next_page:
            next_page = next_page.get('href')
            if next_page:
                await self.scraper_queue.put(ScrapeItem(URL(next_page), scrape_item.parent_title))

    @error_handling_wrapper
    async def post(self, scrape_item: ScrapeItem) -> None:
        """Scrapes an album"""
        async with self.request_limiter:
            soup = await self.client.get_BS4(self.domain, scrape_item.url)

        content = soup.select_one('div[class="flex justify-between items-center"]')
        content_tags = content.select("img")
        content_tags.extend(content.select("source"))

        for selection in content_tags:
            link = URL(selection.get('src'))
            filename, ext = await get_filename_and_ext(link.name)
            await self.handle_file(link, scrape_item, filename, ext)
